# Initrd

> Initrd or initramfs is an initial root filesystem image that is used to
  mount the final root filesystem.

## System boot
\subsection{System boot}

* Bootloader loads the kernel image (and devtree) to RAM
* Kernel is self-extracted and run
* Kernel mounts temporary root file system to `/`
* Modules required for mounting the final rootfs are loaded
* Actions required for mounting the final are performed (user is asked for password to LUKS)
* The `/` is switched to new location and initrd is dropped
* Kernel starts init (sysv, systemd) with PID 1
* Init starts all system services

## What is Initrd?

* Initial RAM Disk provided during boot
* Contents temporarily mounted to /
* Purpose: Load modules and utilities required to mount the rootfs
  * FS modules (LVM, btrfs, ...)
  * Encryption utilities (dm-crypt)
  * network utilities for NFS (dhclient)
* After mounting the rootfs, whole set of modules is available. The
  initrd's memory is released.

## Initrd vs Initramfs
\subsection{Initrd vs Initramfs}

### Initrd

* The older one
* Regular `ramdev` block device
* Requires underlying filesystem $\rightarrow$ has to be compiled in kernel (e.g. ext2)
* dentry, inode for each opened file has to be allocated also in kernel $\rightarrow$ a bit
  higher memory consumption, complexity $\rightarrow$ a bit slower than initramfs

### Initramfs

* Since 2.5.x
* Uses `tmpfs`
* Does not need underlying filesystem
* `tmpfs` support is in kernel, no need for additional modules
* Often called just `initrd`

\subsection{Generating initrd}
## Generating initrd

* Usually each OS distribution provides own script
	* Ubuntu: `update-initramfs`, `mkinitramfs`
	* Archlinux: `mkinitcpio`
		* `mkinitcpio -c /etc/mkinitcpio-custom.conf -g /boot/linux-custom.img`
* Image itself is usally generated based on a config (receipt)

## /etc/mkinitcpio.conf

```sh
MODULES=()
BINARIES=()
FILES=()

# HOOKS
# This is the most important setting in this file.  The H..
# modules and scripts added to the image, and what happen..
# Order is important, and it is recommended that you do n..
# order in which HOOKS are added.  Run 'mkinitcpio -H <ho..
# help on a given hook.
HOOKS=(base udev autodetect modconf block filesystems key..

COMPRESSION="gzip"
```

