\subsection{Docker}
## Docker

![](img/docker_logo.png){width=60%}

* Opensource project for application isolation and deployment
* Building blocks: **images** and **containers**
* **Image** - isolated environment (filesystem) in which an application will run
* **Container** - a process running within an image under Docker engine

## Docker - Motivation

* Application isolation on many levels
  * Filesystem separation (can be easily connected to host FS with bind mounts)
  * Process separation (own set of PIDs, ...)
  * Network isolation
* Applications run natively on host's kernel
  * No performance drop
  * No memory overhead
  * IBM Research on Docker vs VM performance [link][lnk_ibm_docker]
* Fast deployment
  * Images for most frequently used tools/services available on Dockerhub

![](img/mysql.png){width=16%}\ 
![](img/ubuntu.png){width=16%}\ 
![](img/gitlab.png){width=16%}\ 
![](img/nextcloud.png){width=14%}\ 
![](img/python.png){width=10%}\ 
![](img/wordpress.png){width=10%}\ 

## Docker - example

* Run a container with ubuntu image
  ```sh
  docker run -it ubuntu /bin/bash
  ```
  * -i ... interactive
  * -t ... attach to stdin/stdout
* Access host filesystem from container
  ```sh
  docker run -it -v /home/user/wd:/data ubuntu /bin/bash
  ```
  * -v\<host\>:\<guest\> ... bind mount \<host\> directory to \<guest\> within
  	the container

\subsection{Preparing the workspace}
## Preparing the workspace

1. Download the archive to your working directory
[https://files.acrios.com/index.php/s/GAL98g32gdH7HQP][lnk_workshop_stuff]
2. Extract the archive
   ```sh
   tar -xvzf embedded-linux.tar.gz
   cd 01-embedded-linux
   ```
3. Run following command to see all available make targets
   ```sh
   make help
   ```
4. Build the sandbox (may take few minutes to download all stuff)
  run `make sandbox`. This step will download and install all required
  tools to the docker container.

\alert{Next time you run `make sandbox`, only changed files will be re-downloaded.
Use this command to enter the sandbox.}

## Building a kernel for RedPitaya

1. Make sure you are in docker container; working directory `/pitaya`.
2. Run `make kernel-download` to only download the kernel sources, or:
3. Run `make kernel-build` to download and build kernel sources with
   `xilinx_zynq_defconfig`

## Important kernel headers

```C
/* Error checking/converting macros IS_ERR(), PTR_ERR etc. */
#include <linux/err.h>
/* Common kernel macros. KERN_INFO, ALIGN(), ARRAY_SIZE(), abs() */
#include <linux/kernel.h>
/* module_init, module_exit, meta information macros */
#include <linux/module.h>
/* __init and __exit macros */
#include <linux/init.h>
/* string operations */
#include <linux/string.h>
/* Legacy gpio() functions */
#include <linux/gpio.h>
/* gpiod() functions */
#include <linux/gpio/consumer.h>
```

## Important kernel headers

```C
/* Kernel device model */
#include <linux/device.h>
/* Kernel platform device model */
#include <linux/platform_device.h>
/* kmalloc() */
#include <linux/slab.h>
/* Working with device nodes and properties */
#include <linux/of.h>
/* Unified device property interface */
#include <linux/property.h>
/* Bit operations */
#include <linux/bitops.h>
/* struct file_operations, chardev */
#include <linux/fs.h>
/* Userspace access - copy from user, copy to user ... */
#include <linux/uaccess.h>
```
